var setup_overlord = require('overlords_overlord');
var harvester = require('creeps_harvester');

/**
 * Overlord that sets up a room given only a single spawn with a minimum energy to build one harvester.
 * 
 * Setup is considered finished once 3 spawns have been built and at least one creep is maintaining the RCL and at least one harvester is working
 */
const STATE_INIT = 0;
const STATE_CREATE_HARVESTERS = 1;
const STATE_BUILD_SPAWNS = 3;

/**
 * Switches the overlord into the given state
 * @param {int} state 
 */
setup_overlord.prototype.switch_state = function (state) {
    this.state = state;
    console.log("SetupOverlord: switching to state: "+state);
}

/** Checks the room and sets out the tasks to achieve the goal */
setup_overlord.prototype.run = function () {
    //console.log("SetupOverlord: Running \""+this.id+"\"! :) Chilling for now ...");
    //console.log("SetupOverlord: \""+this.id+"\"has the following underlings: ", this.underlings['spawns'], this.underlings['creeps']);
    
    if (this.state === null) {
        this.switch_state(STATE_INIT);
    }
    switch (this.state) {
        case STATE_INIT:
            this.init();
            break;
        case STATE_CREATE_HARVESTERS:
            this.create_harvesters();
            break;
        case STATE_BUILD_SPAWNS:
            console.log("SetupOverlord [ERROR] run(): state not implemented yet!");
            break;
        default:
            console.log("SetupOverlord [ERROR] run(): unknown state: "+this.state);
    }
};

/**
 * Checks that the requirements are given for the setup and then advances the state.
 */
setup_overlord.prototype.init = function() {
    let source_count = Game.spawns[this.underlings["spawns"][0]].room.find(FIND_SOURCES).length;

    this.todo_list = {
        req_harvesters: source_count,
        req_spawns: 3,
        req_RCL_maintainers: 1,
    };

    if (this.underlings['creeps'].length === 0 && this.underlings['spawns'].length === 0) {  // got NOTHING to work with
        console.log("SetupOverlord: ERROR "+this.id+" has nothing! ;W;");
    }
    else {
        this.switch_state(STATE_CREATE_HARVESTERS);
    }
}

/**
 * Function to be run every tick when the overlord aims to create more harvesters for every energy source.
 * Switches to STATE_BUILD_SPAWNS if every source has a harvester assigned.
 */
setup_overlord.prototype.create_harvesters = function() {
    if (this.underlings['creeps'].length === 0 && this.underlings['spawns'].length > 0) {  // first harvester needs to be spawned
        console.log("SetupOverlord: \""+this.id+"\" is trying to create a harvester ...");

        for (let spawn_name of this.underlings['spawns']) {  // check every spawn for energy available
            let spawn = Game.spawns[spawn_name];
            if (spawn.store[RESOURCE_ENERGY] > harvester.min_cost) { // can afford a harvester
                console.log("SetupOverlord: Spawn "+spawn.name+" can afford a harvester! ("+harvester.min_cost+") \\o/");
                let new_harvester_name = harvester.spawn(spawn, this.id+"-"+this.underlings['creeps'].length, this.id);
                this.set_creep_destination_spot(new_harvester_name, spawn);
                this.add_creep(new_harvester_name);
                break;
            }
        }
    }


}

/**
 * Set a creeps destination spot in their memory. This defines where a creep delivers their stuff to.
 * The goto_pos is set to null to indicate to the creep to find their standing spot on their own.
 * 
 * @param {string} creep name of the creep to which this deliver destination will be given
 * @param {RoomPosition} target must be a game object with an id and a pos property
 */
setup_overlord.prototype.set_creep_destination_spot = function(creep, target) {
    console.log("SetupOverlord ["+this.id+"]: Instructing creep \""+creep+"\" to set their destination to "+target.pos);
    Memory.creeps[creep].destination_spot.id = target.id;
    Memory.creeps[creep].destination_spot.structure_pos = target.pos;
    Memory.creeps[creep].destination_spot.goto_pos = null;
};

/**
 * Set a creeps destination spot in their memory. This defines where a creep delivers their stuff to.
 * The goto_pos is set to null to indicate to the creep to find their standing spot on their own.
 * 
 * @param {string} creep name of the creep to which this deliver destination will be given
 * @param {RoomPosition} target must be a game object with an id and a pos property
 */
 setup_overlord.prototype.set_creep_harvest_spot = function(creep, target) {
    console.log("SetupOverlord ["+this.id+"]: Instructing creep \""+creep+"\" to set their destination to "+target.pos);
    Memory.creeps[creep].harvest_spot.id = target.id;
    Memory.creeps[creep].harvest_spot.structure_pos = target.pos;
    Memory.creeps[creep].harvest_spot.goto_pos = null;
};

module.exports = setup_overlord;